local addOnName, TR = ...

-- Locals and map data handling
local TrackResourcesMapFrames = {}

-- WorldMap MapCanvas Setup
local trPinsTemplateName  = "TrackResourcesPinsTemplate"
local uiMapIdItems        = {}
local worldmapPinsPool    = CreateFramePool("FRAME")
local worldmapProvider    = CreateFromMixins(MapCanvasDataProviderMixin)
local worldmapProviderPin = CreateFromMixins(MapCanvasPinMixin)

worldmapPinsPool.parent = WorldMapFrame:GetCanvas()
worldmapPinsPool.creationFunc = function(framePool)
    local frame = CreateFrame(framePool.frameType, nil, framePool.parent)
    frame:SetSize(1, 1)
    return Mixin(frame, worldmapProviderPin)
end
worldmapPinsPool.resetterFunc = function(pinPool, pin)
    FramePool_HideAndClearAnchors(pinPool, pin)
    pin:OnReleased()

    pin.pinTemplate = nil
    pin.owningMap = nil
end

WorldMapFrame.pinPools[trPinsTemplateName] = worldmapPinsPool
WorldMapFrame:AddDataProvider(worldmapProvider)

function worldmapProvider:RemoveAllData()
    self:GetMap():RemoveAllPinsByTemplate(trPinsTemplateName)
end

function worldmapProvider:RefreshAllData(fromOnShow)
    self:RemoveAllData() -- clear all map pins
    TrackResourcesMapFrames = {}

    if not TrackResourcesCfg.settings.showZonemap then return end
    local uiMapID = self:GetMap():GetMapID() -- get current map so we know which we want to see

    if uiMapIdItems[uiMapID] then
        for location, itemData in pairs(uiMapIdItems[uiMapID]) do
            if (TR.isHerbalist and TR.herbs[itemData[1]]) or (TR.isMiner and TR.ores[itemData[1]]) or (TR.isEngineer and TR.motes[itemData[1]]) then
                TrackResourcesMapFrames[itemData] = TrackResourcesMap:CreateIconFrame(itemData)
                self:GetMap():AcquirePin(trPinsTemplateName, TrackResourcesMapFrames[itemData], itemData[4], itemData[5])
            end
        end
    end
end

-- Pins
function worldmapProviderPin:OnLoad()
    self:UseFrameLevelType("PIN_FRAME_LEVEL_AREA_POI")
    self:SetScalingLimits(1, 1.0, 1.2)
end

function worldmapProviderPin:OnAcquired(icon, x, y)
    self:SetPosition(x, y)
    self.icon = icon
    icon:SetParent(self)
    icon:ClearAllPoints()
    icon:SetPoint("CENTER", self, "CENTER")
    icon:Show()
end

function worldmapProviderPin:OnReleased()
    if self.icon then
        self.icon:Hide()
        self.icon:SetParent(UIParent)
        self.icon:ClearAllPoints()
        self.icon = nil
    end
end

-- Main Zone map class
TrackResourcesMap = CreateFrame("FRAME")
TrackResourcesMap:RegisterEvent("PLAYER_LOGIN")
TrackResourcesMap:RegisterEvent("ADDON_LOADED")
TrackResourcesMap:SetScript("OnEvent",
    function (self, event, ...)
        if self[event] then
            return self[event](self, ...) 
        else
            --Debug(event)
            --print(...)
        end
    end
)

function TrackResourcesMap:ADDON_LOADED(loadedAddOnName)
    if loadedAddOnName == addOnName then

    end
end

function TrackResourcesMap:PLAYER_LOGIN()
    uiMapIdItems = {}

    TrackResourcesCfg.settings.showZonemap = TrackResourcesCfg.settings.showZonemap == nil or TrackResourcesCfg.settings.showZonemap
    if not TrackResourcesCfg.settings.showZonemap then return end

    local count = 0
    for uiMapID, resources in pairs(TrackResourcesCfg.db) do
        for itemID, locations in pairs(resources) do
            for location, itemData in pairs(locations) do
                self:AddIcon(uiMapID, location, itemData)
                count = count + 1
            end
        end
    end
    TrackResources:Print("DB Loaded (%d nodes)", count)
end

function TrackResourcesMap:CreateIconFrame(itemData)
    local itemID, textureID, uiMapID, x, y, gatheredCount, gatheredLast, selfGrownHerb = unpack(itemData)
    local icon = CreateFrame("FRAME", nil, nil, BackdropTemplateMixin and "BackdropTemplate")
    icon:SetWidth(TrackResourcesCfg.settings.sizeZonemap)
    icon:SetHeight(TrackResourcesCfg.settings.sizeZonemap)
    icon:SetAlpha(TrackResourcesCfg.settings.alpha or TR.iconSettings.alpha)

    local texture = icon:CreateTexture(nil, "OVERLAY")
    texture:SetTexture(textureID, "CLAMPTOBLACKADDITIVE", nil, TrackResourcesCfg.settings.filterMode)

    icon.itemID = itemID
    icon.itemGatheredCount = gatheredCount
    icon.itemGatheredLast = gatheredLast
    icon.itemSelfGrown = selfGrownHerb

    if TR.motes[itemID] or selfGrownHerb then
        icon:SetWidth(TrackResourcesCfg.settings.sizeZonemap + 1)
        icon:SetHeight(TrackResourcesCfg.settings.sizeZonemap + 1)
        icon:SetBackdrop({
            bgFile = "Interface/Buttons/WHITE8x8",
        })

        if TR.motes[itemID] then
            icon:SetBackdropColor(1, 1, 1, 0.7) -- white/gray
        else
            icon:SetBackdropColor(1, 0, 0, 0.4) --- red
        end

        texture:SetWidth(TrackResourcesCfg.settings.sizeZonemap)
        texture:SetHeight(TrackResourcesCfg.settings.sizeZonemap)
        texture:SetPoint("CENTER")
    else
        texture:SetAllPoints(icon)
    end

    icon:SetScript("OnEnter",
        function(self)
            GameTooltip:Hide()
            GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
            GameTooltip:SetItemByID(self.itemID)
            if self.itemGatheredCount then
                GameTooltip:AddLine(("|cff6666ccTR|r Gathered x%d"):format(self.itemGatheredCount))
            end
            if self.itemGatheredLast then
                GameTooltip:AddLine(("|cff6666ccTR|r Last %s ago"):format(TrackResourcesMap:ElapsedTimeAsString(time() - self.itemGatheredLast)))
            end
            if self.itemSelfGrown then
                GameTooltip:AddLine("|cff6666ccTR|r Obtained from Rich Soil")
            end
            GameTooltip:Show()
        end
    )

    icon:SetScript("OnLeave",
        function(self)
            GameTooltip:Hide()
        end
    )


    return icon
end

function TrackResourcesMap:AddIcon(uiMapID, location, itemData)
    if not uiMapIdItems[uiMapID] then uiMapIdItems[uiMapID] = {} end
    uiMapIdItems[uiMapID][location] = itemData
end

function TrackResourcesMap:Toggle()
    if TrackResourcesCfg.settings.showZonemap then
        TrackResourcesCfg.settings.showZonemap = false
        worldmapProvider:RemoveAllData()
        TrackResourcesMapFrames = {}
        TrackResources:Print("Zone icons are now disabled")
    else
        TrackResourcesCfg.settings.showZonemap = true
        self:PLAYER_LOGIN()
        worldmapProvider:RefreshAllData()
        TrackResources:Print("Zone icons are now enabled")
    end
end

function TrackResourcesMap:UpdateIcons()
    worldmapProvider:RefreshAllData()
end

function TrackResourcesMap:ResetIconCache()
    self:PLAYER_LOGIN()
    worldmapProvider:RefreshAllData()
end

function TrackResourcesMap:ElapsedTimeAsString(epoch)
  local days    = floor(epoch / 86400)
  local hours   = floor(mod(epoch, 86400) / 3600)
  local minutes = floor(mod(epoch,3600) / 60)

  return format("%dd %dh %dm",days, hours, minutes)
end